/**************************************************
 * se8r01.h
 * written by Christof Rieger 2017
 * using basic infromation of Anxinke Corporation
 * Description:
 * Arduino comunicationinterface to the se8r01
/**************************************************/
//Global projekt settings of se8r01 chip
#define SE8R01_Use5ByteAdr  1 //0: 4Byte 1: 5Byte
#define SE8R01_CONT_WAVE    0 //if on use _BV(Se8Bit_CONT_WAVE)
//for selecting speed
#define SE8R01_DR_500K true
//#define SE8R01_DR_1M true
//#define SE8R01_DR_2M true
///////////////////////////////////////////////////////////////////////////////////////
//Needed includes
#include <SPI.h>
#include <avr/pgmspace.h>
///////////////////////////////////////////////////////////////////////////////////////
//Global useabel calculatet variabls and definition
#define SE8R01_ADR_WIDTH    SE8R01_Use5ByteAdr + 4
#if defined(SE8R01_DR_500K)
  #define SE8R01_speed   Se8_500kBit
  #define c_PLL_CTL0  0xE4
  #define c_IF_FREQ   0x1F
  #define c_FDEV      0x14
  #define a_PLL_CTL0  0xE0
  #define a_FDEV      0x14
       
#elif defined(SE8R01_DR_1M)
  #define SE8R01_speed   Se8_1MBit
  #define c_PLL_CTL0  0xE4
  #define c_IF_FREQ   0x1F
  #define c_FDEV      0x14
  #define a_PLL_CTL0  0xE0
  #define a_FDEV      0x14  
      
#elif defined(SE8R01_DR_2M)
  #define SE8R01_speed   Se8_2MBit
  #define c_PLL_CTL0  0xE6
  #define c_IF_FREQ   0x1E
  #define c_FDEV      0x29
  #define a_PLL_CTL0  0xE2
  #define a_FDEV      0x29  
#endif

byte SE8R01_CEq, SE8R01_CSNq, SE8R01_Status;
#define Se8SPI_OPEN   0x80
#define Se8SPI_CLOSE  0x40
#define Se8SPI_CPL    0xC0
#define Se8SPI_ADD    0x00

#define Se8SPI_READ   0x00
#define Se8SPI_WRITE  0x20
///////////////////////////////////////////////////////////////////////////////////////
// BYTE type definition
#define se8r01_H
///////////////////////////////////////////////////////////////////////////////////////
#define Se8_BANK0                  0x00
#define Se8_BANK1                  0x80
//
//-------------------------------------------------------------------------------

//-------------------------------------------------------------------------------
// SE8R01 SPI Commands
//
#define Se8_CMD_READ_REG            0x00            // 000x xxxx Define read command to register
#define Se8_CMD_WRITE_REG           0x20            // 001x xxxx Define write command to register
#define Se8_CMD_RD_RX_PLOAD         0x61            // 0110 0001 Define RX payload register address
#define Se8_CMD_WR_TX_PLOAD         0xA0            // 1010 0000 Define TX payload register address
#define Se8_CMD_FLUSH_TX            0xE1            // 1110 0001 Define flush TX register command
#define Se8_CMD_FLUSH_RX            0xE2            // 1110 0010 Define flush RX register command
#define Se8_CMD_REUSE_TX_PL         0xE3            // 1110 0011 Define reuse TX payload register command
#define Se8_CMD_W_TX_PAYLOAD_NOACK  0xB0            // 1011 0000
#define Se8_CMD_W_ACK_PAYLOAD       0xA8            // 1010 1xxx
#define Se8_CMD_ACTIVATE            0x50            // 0101 0000
#define Se8_CMD_R_RX_PL_WID         0x60            // 0110 0000
#define Se8_CMD_NOP                 0xFF            // 1111 1111 Define No Operation, might be used to read status register
//
//-------------------------------------------------------------------------------

//-------------------------------------------------------------------------------
// SE8R01 registers addresses
//
#define Se8_BK0_CONFIG        0x00            // 'Config' register address
  #define Se8Bit_RX_DR_OFF   6   // 1 blocks IRQ RX-Ready
  #define Se8Bit_TX_DS_OFF   5   // 1 blocks IRQ TX-Ready
  #define Se8Bit_MAX_RT_OFF  4   // 1 blocks IRQ Max Retrys unsuccesfull
  #define Se8Bit_EN_CRC      3   // 1 CRC use ist aktiv
  #define Se8Bit_CRCO        2   // 1 16Bit CRC else 8Bit CRC
  #define Se8Bit_PWR_UP      1   // 1 PowerUp
  #define Se8Bit_PRIM_RX     0   // 1 PRX-Mode 0 PTX-Mode
#define Se8_BK0_EN_AA         0x01            // 'Enable Auto Acknowledgment' register address
  #define Se8Bit_ENAA_P5     5   // 1 Enable Auto Acknwledgement for this Pipe
  #define Se8Bit_ENAA_P4     4
  #define Se8Bit_ENAA_P3     3
  #define Se8Bit_ENAA_P2     2
  #define Se8Bit_ENAA_P1     1
  #define Se8Bit_ENAA_P0     0
#define Se8_BK0_EN_RXADDR     0x02            // 'Enabled RX addresses' register address
  #define Se8Bit_ERX_P5      5   // 1 Enable listenig on this Pipe
  #define Se8Bit_ERX_P4      4
  #define Se8Bit_ERX_P3      3
  #define Se8Bit_ERX_P2      2
  #define Se8Bit_ERX_P1      1
  #define Se8Bit_ERX_P0      0
#define Se8_BK0_SETUP_AW      0x03            // 'Setup address width' register address
  //3: 5 Byts Address for Pipes
  //2: 4 Byts Address for Pipes
#define Se8_BK0_SETUP_RETR    0x04            // 'Setup Auto. Retrans' register address
  #define Se8Nib_ARD         4   // HNibbel 0-15 Auto Retransmit Delay ARD x 256µS
  #define Se8Nib_ARC         0   // LNibbel 0-15 Auto Retransmit Count 0: no Re-Transmit
#define Se8_BK0_RF_CH         0x05            // F0= 2400 + RF_CH[MHz] 2.400GHz to 2.525GHz Europa(0-84)
#define Se8_BK0_RF_SETUP      0x06            // 'RF setup' register address
  #define Se8Bit_CONT_WAVE   7   // 1 Carrier always on
  #define Se8Bit_PA_PWR_3    6
  #define Se8Bit_RF_DR_LOW   5   // 11: 500k  01: 2M  00: 1M
  #define Se8Bit_RF_DR_HIGH  3
  #define Se8Bit_PA_PWR_2    2   // 1111: +5 dbm 1000 0 dbm
  #define Se8Bit_PA_PWR_1    1   // 0100: -6 dbm 0010 -12 dbm
  #define Se8Bit_PA_PWR_0    0   // 0001: -18 dbm
    #define Se8_500kBit      0b00101000   // 500k
    #define Se8_1MBit        0b00000000   // 1M
    #define Se8_2MBit        0b00001000   // 2M
    #define Se8_PA_MIN       0b00000001   // -18 dbm 
    #define Se8_PA_LOW       0b00000010   // -12 dbm
    #define Se8_PA_HIGH      0b00000100   // - 6 dbm
    #define Se8_PA_MAX       0b01000000   //   0 dbm
    #define Se8_PA_OVR       0b01000111   // + 5 dbm
#define Se8_BK0_STATUS        0x07            // 'Status' register address
  #define Se8Bit_BANK        7   // 1 BANK1 read only
  #define Se8Bit_RX_DR       6   // 1 Data reciving ready Write 1 to clear bit
  #define Se8Bit_TX_DS       5   // 1 Data transmitting ready Write 1 to clear bit
  #define Se8Bit_MAX_RT      4   // 1 Maximun TX-retrys reached Write 1 to clear bit
  #define Se8Bit_RX_P_NO_2   3   // 7: RX FIFO Empty 0-5 Data from pipe 0-5
  #define Se8Bit_RX_P_NO_1   2
  #define Se8Bit_RX_P_NO_0   1
  #define Se8Bit_TX_FULL     0   // 1: All TX FIFO are full
#define Se8_BK0_OBSERVE_TX    0x08            // 'Observe TX' register address
  #define Se8Nib_PLOS_CNT    4   // HNibbel 0-15 Count the lost packets. Reset by Write RF_CH
  #define Se8Nib_ARC_CNT     0   // LNibbel 0-15 Last resending trys
#define Se8_BK0_RPD           0x09            // Resiving Carryerlevel as sigend Byte in dBm
#define Se8_BK0_RX_ADDR_P0    0x0A            // 5 Byte RX_Address Pipe 0 LSByte first
#define Se8_BK0_RX_ADDR_P1    0x0B            // 1 Byte RX_Address Pipe 1 LSByte
#define Se8_BK0_RX_ADDR_P2    0x0C            // 1 Byte RX_Address Pipe 2 LSByte
#define Se8_BK0_RX_ADDR_P3    0x0D            // 1 Byte RX_Address Pipe 3 LSByte
#define Se8_BK0_RX_ADDR_P4    0x0E            // 1 Byte RX_Address Pipe 4 LSByte
#define Se8_BK0_RX_ADDR_P5    0x0F            // 1 Byte RX_Address Pipe 5 LSByte
#define Se8_BK0_TX_ADDR       0x10            // 5 Byte TX_Address LSByte first
#define Se8_BK0_RX_PW_P0      0x11            // Recived Bytes in RX-Pipe0
#define Se8_BK0_RX_PW_P1      0x12            // Recived Bytes in RX-Pipe1
#define Se8_BK0_RX_PW_P2      0x13            // Recived Bytes in RX-Pipe2
#define Se8_BK0_RX_PW_P3      0x14            // Recived Bytes in RX-Pipe3
#define Se8_BK0_RX_PW_P4      0x15            // Recived Bytes in RX-Pipe4
#define Se8_BK0_RX_PW_P5      0x16            // Recived Bytes in RX-Pipe5
#define Se8_BK0_FIFO_STATUS   0x17            // 'FIFO Status Register' register address
  #define Se8Bit_TX_REUSE_PL 6   // 1 TX data reused
  #define Se8Bit_TX_FULL     5   // 1 TX FIFO full
  #define Se8Bit_TX_EMPTY    4   // 1 TX FIFO empty
  #define Se8Bit_RX_FULL     1   // 1 RX FIFO full
  #define Se8Bit_RX_EMPTY    0   // 1 RX FIFO empty
#define Se8_BK0_DYNPD         0x1C            // 'Enable dynamic payload length' register address
  #define Se8Bit_DPL_P5      5   // 1 Enable dyn. payload length for this Pipe
  #define Se8Bit_DPL_P4      4
  #define Se8Bit_DPL_P3      3
  #define Se8Bit_DPL_P2      2
  #define Se8Bit_DPL_P1      1
  #define Se8Bit_DPL_P0      0
#define Se8_BK0_FEATURE       0x1D            // 'Feature' register address
  #define Se8Bit_EN_DPL      2   // 1 Enables Dynamic Payload Length
  #define Se8Bit_EN_ACK_PAY  1   // 1 Enables Payload with ACK
  #define Se8Bit_EN_DYN_ACK  0   // 1 Enables the W_TX_PAYLOAD_NOACK command
#define Se8_BK0_SETUP_VALUE   0x1E
  #define Se8Byte_RX_SETUP_VALUE  0  // RX_SETUP time, the time between Standby to RX mode 0-255 µS
  #define Se8Byte_TX_SETUP_VALUE  1  // TX_SETUP time, the time between Standby to TX mode 0-255 µS
  #define Se8Byte_RX_TM_CNT       2  // RX timeout counter 0-255 µS
  #define Se8Byte_REG_MBG_WAIT    3  // Main bandgap wait counter 0-255 µS
  #define Se8Byte_REG_LNA_WAIT    4  // Lna wait counter 0-255 cycle
#define Se8_BK0_PRE_GURD      0x1F
  #define Se8Bit_TAIL_CTL_2 7   // 0-7: Number of repeat bit after the CRC
  #define Se8Bit_TAIL_CTL_1 6
  #define Se8Bit_TAIL_CTL_0 5
  #define Se8Bit_GRD_EN     4   // 1 Enables Pre-Guard
  #define Se8Nib_GRD_CNT    0   // 0-15: 1-16 Number of Pre-Guard bit before preamble
  #define Se8Byte_SPARE_REG 1   // Output to analogue

//SE8R01 Bank1 register
#define Se8_BK1_LINE          0x00
#define Se8_BK1_PLL_CTL0      0x01
#define Se8_BK1_PLL_CTL1      0x02
#define Se8_BK1_CAL_CTL       0x03
#define Se8_BK1_A_CNT_REG     0x04
#define Se8_BK1_B_CNT_REG     0x05
#define Se8_BK1_RESERVED0     0x06
#define Se8_BK1_STATUS        0x07
#define Se8_BK1_STATE         0x08
#define Se8_BK1_CHAN          0x09
#define Se8_BK1_IF_FREQ       0x0A
#define Se8_BK1_AFC_COR       0x0B
#define Se8_BK1_FDEV          0x0C
#define Se8_BK1_DAC_RANGE     0x0D
#define Se8_BK1_DAC_IN        0x0E
#define Se8_BK1_CTUNING       0x0F
#define Se8_BK1_FTUNING       0x10
#define Se8_BK1_RX_CTRL       0x11
#define Se8_BK1_FAGC_CTRL     0x12
#define Se8_BK1_FAGC_CTRL_1   0x13
#define Se8_BK1_DAC_CAL_LOW   0x17
#define Se8_BK1_DAC_CAL_HI    0x18
#define Se8_BK1_RESERVED1     0x19
#define Se8_BK1_DOC_DACI      0x1A
#define Se8_BK1_DOC_DACQ      0x1B
#define Se8_BK1_AGC_CTRL      0x1C
#define Se8_BK1_AGC_GAIN      0x1D
#define Se8_BK1_RF_IVGEN      0x1E
#define Se8_BK1_TEST_PKDET    0x1F
//_BV(RX_DR)
///////////////////////////////////////////////////////////////////////////////////////
//Macros
#define SE8R01Set_IDLESTATE() digitalWrite(SE8R01_CEq, 0)
#define SE8R01Set_ACTIVESTATE() digitalWrite(SE8R01_CEq, 1)

#define SE8R01_FLUSH_TX() SPI_Single(Se8_CMD_FLUSH_TX,0)
#define SE8R01_FLUSH_RX() SPI_Single(Se8_CMD_FLUSH_RX,0)
#define SE8R01_REUSE_TX_PL() SPI_Single(Se8_CMD_REUSE_TX_PL,0)
#define SE8R01Set_ACTIVATE(Val) SPI_Single(Se8_CMD_ACTIVATE,Val)
#define SE8R01Get_STATUS() SPI_Single(Se8_CMD_NOP,0)
#define SE8R01Get_RX_PL_WID() SPI_Single(Se8_CMD_R_RX_PL_WID,0)

#define SE8R01Get_RX_PLOAD(Buf,Cnt) SPI_Buf(Se8_CMD_RD_RX_PLOAD,Buf,(Se8SPI_CPL|Se8SPI_READ)|(Cnt&0x1F)) 

#define SE8R01Set_TX_PLOAD(Buf,Cnt) SPI_Buf(Se8_CMD_WR_TX_PLOAD,Buf,(Se8SPI_CPL|Se8SPI_WRITE)|(Cnt&0x1F))
#define SE8R01Set_TX_PAYLOAD_NOACK(Buf,Cnt) SPI_Buf(Se8_CMD_W_TX_PAYLOAD_NOACK,Buf,(Se8SPI_CPL|Se8SPI_WRITE)|(Cnt&0x1F))
#define SE8R01Set_ACK_PAYLOAD(Pipe,Buf,Cnt) SPI_Buf(Se8_CMD_W_ACK_PAYLOAD|Pipe,Buf,(Se8SPI_CPL|Se8SPI_WRITE)|(Cnt&0x1F))

#define SE8R01Set_CONFIG(Val) SPI_Single(Se8_CMD_WRITE_REG|Se8_BK0_CONFIG,Val)
#define SE8R01Set_EN_AA(Val) SPI_Single(Se8_CMD_WRITE_REG|Se8_BK0_EN_AA,Val)
#define SE8R01Set_EN_RXADDR(Val) SPI_Single(Se8_CMD_WRITE_REG|Se8_BK0_EN_RXADDR,Val)
#define SE8R01Set_SETUP_RETR(Val) SPI_Single(Se8_CMD_WRITE_REG|Se8_BK0_SETUP_RETR,Val)
#define SE8R01Set_RF_CH(Val) SPI_Single(Se8_CMD_WRITE_REG|Se8_BK0_RF_CH,Val)
#define SE8R01Set_STATUS(Val) SPI_Single(Se8_CMD_WRITE_REG|Se8_BK0_STATUS,Val)
#define SE8R01Set_RX_ADDR_P0(Val) SPI_Single(Se8_CMD_WRITE_REG|Se8_BK0_RX_ADDR_P0,Val)
#define SE8R01Set_RX_ADDR_P1(Val) SPI_Single(Se8_CMD_WRITE_REG|Se8_BK0_RX_ADDR_P1,Val)
#define SE8R01Set_RX_ADDR_P2(Val) SPI_Single(Se8_CMD_WRITE_REG|Se8_BK0_RX_ADDR_P2,Val)
#define SE8R01Set_RX_ADDR_P3(Val) SPI_Single(Se8_CMD_WRITE_REG|Se8_BK0_RX_ADDR_P3,Val)
#define SE8R01Set_RX_ADDR_P4(Val) SPI_Single(Se8_CMD_WRITE_REG|Se8_BK0_RX_ADDR_P4,Val)
#define SE8R01Set_RX_ADDR_P5(Val) SPI_Single(Se8_CMD_WRITE_REG|Se8_BK0_RX_ADDR_P5,Val)
#define SE8R01Set_TX_ADDR(Val) SPI_Single(Se8_CMD_WRITE_REG|Se8_BK0_TX_ADDR,Val)
#define SE8R01Set_RX_PW_P0(Val) SPI_Single(Se8_CMD_WRITE_REG|Se8_BK0_RX_PW_P0,Val)
#define SE8R01Set_RX_PW_P1(Val) SPI_Single(Se8_CMD_WRITE_REG|Se8_BK0_RX_PW_P1,Val)
#define SE8R01Set_RX_PW_P2(Val) SPI_Single(Se8_CMD_WRITE_REG|Se8_BK0_RX_PW_P2,Val)
#define SE8R01Set_RX_PW_P3(Val) SPI_Single(Se8_CMD_WRITE_REG|Se8_BK0_RX_PW_P3,Val)
#define SE8R01Set_RX_PW_P4(Val) SPI_Single(Se8_CMD_WRITE_REG|Se8_BK0_RX_PW_P4,Val)
#define SE8R01Set_RX_PW_P5(Val) SPI_Single(Se8_CMD_WRITE_REG|Se8_BK0_RX_PW_P5,Val)
#define SE8R01Set_DYNPD(Val) SPI_Single(Se8_CMD_WRITE_REG|Se8_BK0_DYNPD,Val)
#define SE8R01Set_FEATURE(Val) SPI_Single(Se8_CMD_WRITE_REG|Se8_BK0_FEATURE,Val)

#define SE8R01Init_PA_PWR(Val) SPI_Single(Se8_CMD_WRITE_REG|Se8_BK0_RF_SETUP,((_BV(Se8Bit_CONT_WAVE)*SE8R01_CONT_WAVE)|SE8R01_speed)|Val)
#define SE8R01Init_ADDR(FirstRX,FirstTX,Main) SE8R01Set_ADDR_F((byte*)(&FirstRX),(byte*)(&FirstTX),(byte*)(&Main))

#define SE8R01Set_RX_ADDR_MASTER(Buf) SPI_Buf(Se8_CMD_WRITE_REG|Se8_BK0_RX_ADDR_P0,Buf,(Se8SPI_CPL|Se8SPI_WRITE)|SE8R01_ADR_WIDTH)
#define SE8R01Set_TX_ADDR_MASTER(Buf) SPI_Buf(Se8_CMD_WRITE_REG|Se8_BK0_TX_ADDR,Buf,(Se8SPI_CPL|Se8SPI_WRITE)|SE8R01_ADR_WIDTH)

#define SE8R01Get_R_STATUS() SPI_Single(Se8_CMD_READ_REG|Se8_BK0_STATUS,0)
#define SE8R01Get_OBSERVE_TX() SPI_Single(Se8_CMD_READ_REG|Se8_BK0_OBSERVE_TX,0)
#define SE8R01Get_RPD() SPI_Single(Se8_CMD_READ_REG|Se8_BK0_RPD,0)
#define SE8R01Get_RX_PW_P0() SPI_Single(Se8_CMD_READ_REG|Se8_BK0_RX_PW_P0,0)
#define SE8R01Get_RX_PW_P1() SPI_Single(Se8_CMD_READ_REG|Se8_BK0_RX_PW_P1,0)
#define SE8R01Get_RX_PW_P2() SPI_Single(Se8_CMD_READ_REG|Se8_BK0_RX_PW_P2,0)
#define SE8R01Get_RX_PW_P3() SPI_Single(Se8_CMD_READ_REG|Se8_BK0_RX_PW_P3,0)
#define SE8R01Get_RX_PW_P4() SPI_Single(Se8_CMD_READ_REG|Se8_BK0_RX_PW_P4,0)
#define SE8R01Get_RX_PW_P5() SPI_Single(Se8_CMD_READ_REG|Se8_BK0_RX_PW_P5,0)
#define SE8R01Get_FIFO_STATUS() SPI_Single(Se8_CMD_READ_REG|Se8_BK0_FIFO_STATUS,0)

