/**************************************************
 * se8r01.ino
 * written by Christof Rieger 2017
 * using basic infromation of Anxinke Corporation
 * Description:
 * Arduino comunicationinterface to the se8r01
/**************************************************
 * Arrays with basic initial data
/**************************************************/
const PROGMEM byte cal_data[] = {0xB0,
                                0x01,Se8_CMD_WRITE_REG|Se8_BK0_CONFIG,0x03,
                                0x01,Se8_CMD_WRITE_REG|Se8_BK0_RF_CH,0x32,
                                0x01,Se8_CMD_WRITE_REG|Se8_BK0_RF_SETUP,SE8R01_speed|Se8_PA_MAX,
                                0x01,Se8_CMD_WRITE_REG|Se8_BK0_PRE_GURD,0x77,
                                0xB1,
                                0x04,Se8_CMD_WRITE_REG|Se8_BK1_PLL_CTL0,0x40,0x00,0x10,c_PLL_CTL0,
                                0x05,Se8_CMD_WRITE_REG|Se8_BK1_CAL_CTL,0x20,0x08,0x50,0x40,0x50,
                                0x03,Se8_CMD_WRITE_REG|Se8_BK1_IF_FREQ,0x00,0x00,c_IF_FREQ,
                                0x01,Se8_CMD_WRITE_REG|Se8_BK1_FDEV,c_FDEV,
                                0x01,Se8_CMD_WRITE_REG|Se8_BK1_DAC_CAL_LOW,0x00,
                                0x01,Se8_CMD_WRITE_REG|Se8_BK1_DAC_CAL_HI,0x7F,
                                0x04,Se8_CMD_WRITE_REG|Se8_BK1_AGC_GAIN,0x02,0xC1,0xEB,0x1C,
                                0x04,Se8_CMD_WRITE_REG|Se8_BK1_RF_IVGEN,0x97,0x64,0x00,0x81,
                                0xB0,
                                0xED};

const PROGMEM byte an1_data[] = {0x05,Se8_CMD_WRITE_REG|Se8_BK0_SETUP_VALUE,0x28,0x32,0x80,0x90 - SE8R01_Use5ByteAdr * 0x80,0x00,
                                0xED};

const PROGMEM byte ana_data[] = {0xB1,
                                0x04,Se8_CMD_WRITE_REG|Se8_BK1_PLL_CTL0,0x40,0x01,0x30,a_PLL_CTL0,
                                0x05,Se8_CMD_WRITE_REG|Se8_BK1_CAL_CTL,0x29,0x89,0x55,0x40,0x50,
                                0x01,Se8_CMD_WRITE_REG|Se8_BK1_FDEV,a_FDEV,
                                0x04,Se8_CMD_WRITE_REG|Se8_BK1_RX_CTRL,0x55 - SE8R01_Use5ByteAdr * 4,0xC2,0x09,0xAC,
                                0x04,Se8_CMD_WRITE_REG|Se8_BK1_FAGC_CTRL_1,0x00,0x14,0x08,0x29,
                                0x04,Se8_CMD_WRITE_REG|Se8_BK1_AGC_GAIN,0x02,0xC1,0xCB,0x1C,
                                0x04,Se8_CMD_WRITE_REG|Se8_BK1_RF_IVGEN,0x97,0x64,0x00,0x01,
                                0x04,Se8_CMD_WRITE_REG|Se8_BK1_TEST_PKDET,0x2A,0x04,0x00,0x7D,
                                0xB0,
                                0x01,Se8_CMD_WRITE_REG|Se8_BK0_SETUP_AW,SE8R01_Use5ByteAdr + 2,
                                0xED};

byte SE8R01_ListenPipe = 0;

/**************************************************
 * Function: SE8R01_BEGIN();
 * Description:
 * Set SPI and setup the se8r01
 * flash led one time,chip enable(ready to TX or RX Mode),
/**************************************************/
void SE8R01_BEGIN(byte pCE, byte pCSN){
  SE8R01_CEq = pCE;
  SE8R01_CSNq = pCSN;
  //pinMode(IRQq, INPUT);
  pinMode(SE8R01_CEq,  OUTPUT);
  pinMode(SE8R01_CSNq, OUTPUT);
  digitalWrite(SE8R01_CEq, 0);      // chip disable
  digitalWrite(SE8R01_CSNq, 1);     // Spi disable
  SPI.begin();
  SPI.beginTransaction(SPISettings(8000000, MSBFIRST, SPI_MODE0));
  SPI_BlockSend(cal_data);
  for(byte i=0;i<2; i++){            // 2xInit
        digitalWrite(SE8R01_CEq, 1); 
        delayMicroseconds(30);
        digitalWrite(SE8R01_CEq, 0);  
        delay(50);                   // delay 50ms waitting for calibaration.
  }
  SPI_BlockSend(an1_data);   
  delay(2);
  SPI_BlockSend(ana_data);
}
/**************************************************
 * Function: SPI_Single();
 * 
 * Description:
 * Transfares one Data for a cammand
/**************************************************/
byte SPI_Single(byte CMD,byte UniData){
  boolean DoOne = true;
  byte reg_val = 0;
  if((CMD & 0xE0)==0xE0){DoOne = false;} 
  if((CMD < 0x20)|(CMD == Se8_CMD_R_RX_PL_WID)){UniData = 0;}   
  digitalWrite(SE8R01_CSNq, 0);  // CSN low, init SPI transaction
  SE8R01_Status = SPI.transfer(CMD);         // CMD
  if(DoOne){reg_val = SPI.transfer(UniData);}     
  digitalWrite(SE8R01_CSNq, 1);                   // CSN high again
  if(((CMD > 0x19)&&(CMD < 0x40))|(CMD == Se8_CMD_ACTIVATE)){reg_val = 0;}
  return(reg_val);                  // return nRF24L01 status unsigned char
}

/**************************************************
 * Function: SPI_Buf();
 * 
 * Description:
 * Reads and writes with Open and Close Databytes
 * It is posible to transfare bytes by multicalls 
/**************************************************/
void SPI_Buf(byte reg, byte *pBuf, byte code){
  byte bytes = code & 0x1F;
  if (bytes == 0){bytes = 0x20;}
  if ((code & Se8SPI_OPEN) == Se8SPI_OPEN){
    digitalWrite(SE8R01_CSNq, 0);                  // Set CSN low, init SPI tranaction
    SE8R01_Status = SPI.transfer(reg);             // Select register to write to and read status unsigned char
  }  
  for(byte i=0;i<bytes;i++){
    if((code & Se8SPI_WRITE) == Se8SPI_WRITE){
      SPI.transfer(pBuf[i]);
    }else{
      pBuf[i] = SPI.transfer(0);
    }  
  }
  if ((code & Se8SPI_CLOSE) == Se8SPI_CLOSE){digitalWrite(SE8R01_CSNq, 1);} // Set CSN high again
}
/**************************************************
 * Function: SPI_BlockSend();
 * 
 * Description:
 * Sends an array of multible configdata 
 * to the se8r01
/**************************************************/
void SPI_BlockSend(byte *pBuf){
  byte i = 255;
  while(pgm_read_byte_near(++i + pBuf) != 0xED){
    switch (pgm_read_byte_near(i + pBuf)){
      case 0xB0:rf_switch_bank(Se8_BANK0);break;
      case 0xB1:rf_switch_bank(Se8_BANK1);break;
      default:{
        digitalWrite(SE8R01_CSNq, 0);
        byte j = i + pgm_read_byte_near(i + pBuf) +1;
        while(j > i){
          SPI.transfer(pgm_read_byte_near(++i + pBuf));
        }
        digitalWrite(SE8R01_CSNq, 1);
      }
    }
  }  
}
/**************************************************/
void rf_switch_bank(byte bankindex){
    if((SE8R01Get_R_STATUS()&0x80)!=bankindex){SE8R01Set_ACTIVATE(0x53);}
}

void SE8R01Set_ADDR_F(byte *firstRX, byte *firstTX, byte *main){
  SPI_Buf(Se8_CMD_WRITE_REG|Se8_BK0_RX_ADDR_P0,firstRX,(Se8SPI_OPEN|Se8SPI_WRITE)|1);
  SPI_Buf(Se8_CMD_WRITE_REG|Se8_BK0_RX_ADDR_P0,main,(Se8SPI_CLOSE|Se8SPI_WRITE)|(SE8R01_ADR_WIDTH-1));
  SPI_Buf(Se8_CMD_WRITE_REG|Se8_BK0_TX_ADDR,firstTX,(Se8SPI_OPEN|Se8SPI_WRITE)|1);
  SPI_Buf(Se8_CMD_WRITE_REG|Se8_BK0_TX_ADDR,main,(Se8SPI_CLOSE|Se8SPI_WRITE)|(SE8R01_ADR_WIDTH-1));
}

